﻿#if UNITY_EDITOR

using UnityEditor;

namespace Vanilla.Core.Editor
{
    public class VanillaCoreHierarchyMenu : VanillaHierarchyMenuBase
    {
        // ----------------------------------------------------------------------------------------------- Constants //

        #region Constants

        private const string ToolsMenuPath = Constants.CoreHierarchyMenuPath + ToolsSubMenu;

        private const string PrimitivesMenuPath =
            Constants.CoreHierarchyMenuPath + PrimitivesSubMenu;

        private const string LettersMenuPath =
            Constants.CoreHierarchyMenuPath + LettersSubMenu + Letter;

        private const string ToolsPrefabPath = Constants.PrefabsPath + ToolsSubMenu;
        private const string PrimitivesPrefabPath = Constants.PrefabsPath + PrimitivesSubMenu;
        private const string LettersPrefabPath = Constants.PrefabsPath + LettersSubMenu;

        private const string ToolsSubMenu = "Tools/";
        private const string PrimitivesSubMenu = "Primitives/";
        private const string LettersSubMenu = "Letters/";

        private const string Boxlet = "Boxlet";
        private const string Capsule = "Capsule";
        private const string Circle = "Circle";
        private const string Cone = "Cone";
        private const string Dome = "Dome";
        private const string Donut = "Donut";
        private const string Knot = "Knot";
        private const string Pyramid = "Pyramid";
        private const string Ramp = "Ramp";
        private const string Sphere = "Sphere";
        private const string Tube = "Tube";

        private const string Letter = "Letter ";

        private const string LowPath = "/Low";
        private const string MedPath = "/Med";
        private const string HighPath = "/High";

        private const string LowQuality = " [Low]";
        private const string MedQuality = " [Med]";
        private const string HighQuality = " [High]";

        private const string Inverted = " [Inverted]";

        private const int Priority = 10;

        #endregion

        // --------------------------------------------------------------------------------------------------- Tools //

        #region Tools

        [MenuItem(itemName: ToolsMenuPath + "Transform Compass", isValidateFunction: false, priority: Priority)]
        public static void CreateTransformCompass(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: ToolsPrefabPath,
                prefabName: "Transform Compass");
        }

        #endregion

        // ---------------------------------------------------------------------------------------------- Primitives //

        #region Primitives

        // Boxlet //

        [MenuItem(itemName: PrimitivesMenuPath + Boxlet + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateBoxletLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Boxlet + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Boxlet + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateBoxletMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Boxlet + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Boxlet + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateBoxletHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Boxlet + HighQuality);
        }

        // Capsule //

        [MenuItem(itemName: PrimitivesMenuPath + Capsule + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCapsuleLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Capsule + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Capsule + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCapsuleMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Capsule + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Capsule + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCapsuleHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Capsule + HighQuality);
        }

        // Circle //

        [MenuItem(itemName: PrimitivesMenuPath + Circle + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCircleLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Circle + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Circle + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCircleMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Circle + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Circle + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateCircleHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Circle + HighQuality);
        }

        // Cone //

        [MenuItem(itemName: PrimitivesMenuPath + Cone + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateConeLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Cone + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Cone + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateConeMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Cone + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Cone + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateConeHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Cone + HighQuality);
        }

        // Dome //

        [MenuItem(itemName: PrimitivesMenuPath + Dome + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Dome + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Dome + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + HighQuality);
        }

        // Dome - Inverted //

        [MenuItem(itemName: PrimitivesMenuPath + Dome + LowPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeLowInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + LowQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Dome + MedPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeMedInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + MedQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Dome + HighPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateDomeHighInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + HighQuality + Inverted);
        }

        // Donut //

        [MenuItem(itemName: PrimitivesMenuPath + Donut + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDonutLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Donut + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDonutMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Donut + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateDonutHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Donut + HighQuality);
        }

        // Knot //

        [MenuItem(itemName: PrimitivesMenuPath + Knot + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateKnotHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Knot + HighQuality);
        }

        // Pyramid //

        [MenuItem(itemName: PrimitivesMenuPath + Pyramid + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreatePyramidLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Pyramid + LowQuality);
        }

        // Ramp - Inverted //

        [MenuItem(itemName: PrimitivesMenuPath + Ramp + LowPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateRampLowInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Ramp + LowQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Ramp + MedPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateRampMedInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Ramp + MedQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Ramp + HighPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateRampHighInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Ramp + HighQuality + Inverted);
        }

        // Sphere //

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + HighQuality);
        }

        // Sphere - Inverted //

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + LowPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereLowInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + LowQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + MedPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereMedInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + MedQuality + Inverted);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Sphere + HighPath + Inverted, isValidateFunction: false, priority: Priority)]
        public static void CreateSphereHighInverted(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Sphere + HighQuality + Inverted);
        }

        // Tube //

        [MenuItem(itemName: PrimitivesMenuPath + Tube + LowPath, isValidateFunction: false, priority: Priority)]
        public static void CreateTubeLow(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Tube + LowQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Tube + MedPath, isValidateFunction: false, priority: Priority)]
        public static void CreateTubeMed(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Tube + MedQuality);
        }

        [MenuItem(itemName: PrimitivesMenuPath + Tube + HighPath, isValidateFunction: false, priority: Priority)]
        public static void CreateTubeHigh(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: PrimitivesPrefabPath,
                prefabName: Tube + HighQuality);
        }

        #endregion

        // ------------------------------------------------------------------------------------------------- Letters //

        #region Letters

        [MenuItem(itemName: LettersMenuPath + "V", isValidateFunction: false, priority: Priority)]
        public static void CreateLetterV(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: LettersPrefabPath,
                prefabName: Letter + "V");
        }

        [MenuItem(itemName: LettersMenuPath + "A", isValidateFunction: false, priority: Priority)]
        public static void CreateLetterA(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: LettersPrefabPath,
                prefabName: Letter + "A");
        }

        [MenuItem(itemName: LettersMenuPath + "N", isValidateFunction: false, priority: Priority)]
        public static void CreateLetterN(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: LettersPrefabPath,
                prefabName: Letter + "N");
        }

        [MenuItem(itemName: LettersMenuPath + "I", isValidateFunction: false, priority: Priority)]
        public static void CreateLetterI(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: LettersPrefabPath,
                prefabName: Letter + "I");
        }

        [MenuItem(itemName: LettersMenuPath + "L", isValidateFunction: false, priority: Priority)]
        public static void CreateLetterL(MenuCommand menuCommand)
        {
            CreateInstance(
                menuCommand: menuCommand,
                pathToPrefab: LettersPrefabPath,
                prefabName: Letter + "L");
        }

        #endregion
    }
}

#endif